const MODULE_NAME = 'PARTNER-LISTENER.ROUTERS.PRODUCT-TREE';

const RATE_LIMIT_MESSAGE = 'Rate limited. Cobalah satu menit lagi!\n';
const RATE_LIMIT_MAX = 2;
const RATE_LIMIT_WINDOW_MS = 60 * 1000;

const axios = require('axios').default;
const express = require('express');
const urlJoin = require('join-path');
const expressRateLimit = require('express-rate-limit');

const coreUrl = require('komodo-sdk/core-url');
const logger = require('tektrans-logger');

const ipv6ToIpv4 = require('../../ipv6-to-ipv4');
const getFromBodyQsParams = require('../../get-from-body-qs-params');

const CORE_ENDPOINT = urlJoin(coreUrl, '/product-tree');

const router = express.Router();
module.exports = router;

const rateLimit = expressRateLimit({
    windowMs: RATE_LIMIT_WINDOW_MS,
    max: RATE_LIMIT_MAX,
    message: RATE_LIMIT_MESSAGE,
    keyGenerator: (req, res) => res.locals && res.locals.terminalName,
    // handler: (req, res, next, opts) => {
    //     onRateLimited(req, res, 'ip', opts);
    // },
});

const extractTerminalName = (req, res, next) => {
    const terminalNameWithoutIp = (getFromBodyQsParams(req, 'terminal_name') || '').toString().trim();
    res.locals.terminalName = `${terminalNameWithoutIp}@${ipv6ToIpv4(req.ip)}`;
    next();
};

const mainHandler = async (req, res) => {
    const { xid } = res.locals;

    const { terminalName } = res.locals;
    const password = getFromBodyQsParams(req, 'password');

    try {
        const result = await axios.get(CORE_ENDPOINT, {
            params: {
                terminal_name: terminalName,
                password,
            },
        });

        if (!result || !result.data) {
            const e = new Error(`${MODULE_NAME} B7555411: Invalid result from CORE`);
            throw e;
        }

        result.httpgetx_xid = xid;
        res.json(result.data);
    } catch (e) {
        logger.warn(`${MODULE_NAME} 5AA99A06: Exception`, {
            xid,
            eCode: e.code,
            eMessage: e.message || e,
        });

        res.json({
            httpgetx_xid: xid,
            ts: new Date(),
            error: e.message || e,
        });
    }
};

router.get('/', extractTerminalName, rateLimit, mainHandler);
